import requests
from typing import List

from ._common.retry import retry_on_network_error


class Sender:
    def __init__(self, robot_endpoint: str, robot_ak: str, retry: int = 3):
        self.robot_endpoint = robot_endpoint
        self.robot_ak = robot_ak
        self.retry = retry

    @retry_on_network_error()
    def send_robot_with_at(self, group_id: List[int], message: str, at_user_ids: List[str] = None,
                           at_all: bool = False):
        """
        发送消息并@指定用户或@全体成员
        :param group_id: 群聊id列表
        :param message: 要发送的消息内容
        :param at_user_ids: 要at的用户id字符串列表
        :param at_all: 是否@全体成员,True时会忽略at_user_ids
        :return:
        """
        params = {
            "access_token": self.robot_ak
        }
        message = {
            "message": {
                "header": {"toid": group_id},
                "body": [
                    {"content": message, "type": "TEXT"},
                    {"atuserids": at_user_ids, "type": "AT", "atall": at_all}
                ]
            }
        }
        response = requests.post(self.robot_endpoint, params=params, json=message)
        response.raise_for_status()
        return response.json()

    @staticmethod
    def call_ali(phone_number: str, message: str):
        from ._call.Ali import CallUtil
        return CallUtil.call(phone_number, message)
    @staticmethod
    def get_call_status_ali(call_id: str, timestamp: int):
        from ._call.Ali import CallUtil
        return CallUtil.get_call_status(call_id, timestamp)





import json
import os
import time
from datetime import datetime

from alibabacloud_dyvmsapi20170525.client import Client as Dyvmsapi20170525Client
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_dyvmsapi20170525 import models as dyvmsapi_20170525_models
from alibabacloud_tea_util import models as util_models
from alibabacloud_tea_util.client import Client as UtilClient
from .._common.logger import logger

class CallUtil:
    def __init__(self):
        pass

    @staticmethod
    def create_client() -> Dyvmsapi20170525Client:
        """
        使用AK&SK初始化账号Client
        @return: Client
        @throws Exception
        """
        # 工程代码泄露可能会导致 AccessKey 泄露，并威胁账号下所有资源的安全性。以下代码示例仅供参考。
        # 建议使用更安全的 STS 方式，更多鉴权访问方式请参见：https://help.aliyun.com/document_detail/378659.html。
        config = open_api_models.Config(
            # 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_ID。,
            access_key_id=os.environ['ALIBABA_CLOUD_ACCESS_KEY_ID'],
            # 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_SECRET。,
            access_key_secret=os.environ['ALIBABA_CLOUD_ACCESS_KEY_SECRET']
        )
        # Endpoint 请参考 https://api.aliyun.com/product/Dyvmsapi
        config.endpoint = f'dyvmsapi.aliyuncs.com'
        return Dyvmsapi20170525Client(config)

    @staticmethod
    def call(
        called_number: str, message: str, called_show_number : str = '05923396264'
    ) -> None:
        client = CallUtil.create_client()
        ttl_param = {"info": message}
        single_call_by_tts_request = dyvmsapi_20170525_models.SingleCallByTtsRequest(
            called_show_number=called_show_number,
            called_number=called_number,
            tts_code='TTS_305560006',
            tts_param=str(ttl_param)
        )
        runtime = util_models.RuntimeOptions()
        try:
            resp = client.single_call_by_tts_with_options(single_call_by_tts_request, runtime)
            resp_str = UtilClient.to_jsonstring(resp)
            logger.info(f"call response: {resp_str}")
            json_data = json.loads(resp_str)
            if json_data['statusCode'] != 200:
                raise Exception(f"status code is {json_data['statusCode']}")
            if json_data['body']['Code'] != 'OK':
                raise Exception(f"code is {json_data['body']['Code']} message is {json_data['body']['Message']}")
            call_id = json_data['body']['CallId']
            time_str = json_data['headers']['date']
            dt = datetime.strptime(time_str, "%a, %d %b %Y %H:%M:%S %Z")
            timestamp = time.mktime(dt.timetuple())
            return call_id, int(timestamp * 1000)
        except Exception as error:
            raise error

    @staticmethod
    def get_call_status(call_id: str, timestamp: int):
        client = CallUtil.create_client()
        query_call_detail_by_call_id_request = dyvmsapi_20170525_models.QueryCallDetailByCallIdRequest(
            call_id=call_id,
            prod_id=11000000300006,
            query_date=timestamp
        )
        runtime = util_models.RuntimeOptions()
        try:
            resp = client.query_call_detail_by_call_id_with_options(query_call_detail_by_call_id_request, runtime)
            resp_str = UtilClient.to_jsonstring(resp)
            logger.info(f"get call status response: {resp_str}")
            json_data = json.loads(resp_str)
            if json_data['statusCode'] != 200:
                raise Exception(f"status code is {json_data['statusCode']}")
            if json_data['body']['Code'] != 'OK':
                raise Exception(f"code is {json_data['body']['Code']} message is {json_data['body']['Message']}")
            result = json.loads(json_data['body']['Data'])
            return result['state'], result['stateDesc']
        except Exception as error:
            raise error




from functools import wraps
from tenacity import retry, stop_after_attempt, wait_exponential, retry_if_exception_type
import requests


def retry_on_network_error():
    """
    返回一个装饰器，用于类方法，自动从 self.retry 获取重试次数
    """
    def decorator(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            # 动态创建带 self.retry 的重试策略
            retrier = retry(
                stop=stop_after_attempt(self.retry),
                wait=wait_exponential(multiplier=1, min=1, max=10),
                retry=retry_if_exception_type((
                    requests.exceptions.Timeout,
                    requests.exceptions.ConnectionError,
                    requests.exceptions.HTTPError  # 可选：包含 5xx 等
                ))
            )
            # 应用重试到原函数
            return retrier(func)(self, * args, **kwargs)
        return wrapper
    return decorator